@@ -20,18 +20,17 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-__version__ = '2.0.1'
+__version__ = '2.1.0'
 __author__ = 'Raphaël Barrois <raphael.barrois+fboy@polytechnique.org>'
 
+
 from .base import (
     Factory,
     BaseDictFactory,
     DictFactory,
     BaseListFactory,
     ListFactory,
-    MogoFactory,
     StubFactory,
-    DjangoModelFactory,
 
     BUILD_STRATEGY,
     CREATE_STRATEGY,
@@ -39,6 +38,9 @@ from .base import (
     use_strategy,
 )
 
+from .mogo import MogoFactory
+from .django import DjangoModelFactory
+
 from .declarations import (
     LazyAttribute,
     Iterator,
