@@ -20,7 +20,12 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+import logging
+
 from . import containers
+from . import utils
+
+logger = logging.getLogger('factory.generate')
 
 # Strategies
 BUILD_STRATEGY = 'build'
@@ -239,6 +244,21 @@ class BaseFactory(object):
     # List of attributes that should not be passed to the underlying class
     FACTORY_HIDDEN_ARGS = ()
 
+    @classmethod
+    def reset_sequence(cls, value=None, force=False):
+        """Reset the sequence counter."""
+        if cls._base_factory:
+            if force:
+                cls._base_factory.reset_sequence(value=value)
+            else:
+                raise ValueError(
+                    "Cannot reset the sequence of a factory subclass. "
+                    "Please call reset_sequence() on the root factory, "
+                    "or call reset_sequence(forward=True)."
+                )
+        else:
+            cls._next_sequence = value
+
     @classmethod
     def _setup_next_sequence(cls):
         """Set up an initial sequence value for Sequence attributes.
@@ -285,7 +305,13 @@ class BaseFactory(object):
         force_sequence = None
         if extra:
             force_sequence = extra.pop('__sequence', None)
-        return containers.AttributeBuilder(cls, extra).build(
+        log_ctx = '%s.%s' % (cls.__module__, cls.__name__)
+        logger.debug('BaseFactory: Preparing %s.%s(extra=%r)',
+            cls.__module__,
+            cls.__name__,
+            extra,
+        )
+        return containers.AttributeBuilder(cls, extra, log_ctx=log_ctx).build(
             create=create,
             force_sequence=force_sequence,
         )
@@ -323,6 +349,11 @@ class BaseFactory(object):
         # Extract *args from **kwargs
         args = tuple(kwargs.pop(key) for key in cls.FACTORY_ARG_PARAMETERS)
 
+        logger.debug('BaseFactory: Generating %s.%s(%s)',
+            cls.__module__,
+            cls.__name__,
+            utils.log_pprint(args, kwargs),
+        )
         if create:
             return cls._create(target_class, *args, **kwargs)
         else:
@@ -349,9 +380,8 @@ class BaseFactory(object):
         # Handle post-generation attributes
         results = {}
         for name, decl in sorted(postgen_declarations.items()):
-            extracted, extracted_kwargs = postgen_attributes[name]
-            results[name] = decl.call(obj, create, extracted,
-                    **extracted_kwargs)
+            extraction_context = postgen_attributes[name]
+            results[name] = decl.call(obj, create, extraction_context)
 
         cls._after_postgeneration(obj, create, results)
 
@@ -554,76 +584,6 @@ class StubFactory(Factory):
         raise UnsupportedStrategy()
 
 
-class DjangoModelFactory(Factory):
-    """Factory for Django models.
-
-    This makes sure that the 'sequence' field of created objects is a new id.
-
-    Possible improvement: define a new 'attribute' type, AutoField, which would
-    handle those for non-numerical primary keys.
-    """
-
-    ABSTRACT_FACTORY = True
-    FACTORY_DJANGO_GET_OR_CREATE = ()
-
-    @classmethod
-    def _get_manager(cls, target_class):
-        try:
-            return target_class._default_manager   # pylint: disable=W0212
-        except AttributeError:
-            return target_class.objects
-
-    @classmethod
-    def _setup_next_sequence(cls):
-        """Compute the next available PK, based on the 'pk' database field."""
-
-        model = cls._associated_class  # pylint: disable=E1101
-        manager = cls._get_manager(model)
-
-        try:
-            return 1 + manager.values_list('pk', flat=True
-                ).order_by('-pk')[0]
-        except IndexError:
-            return 1
-
-    @classmethod
-    def _create(cls, target_class, *args, **kwargs):
-        """Create an instance of the model, and save it to the database."""
-        manager = cls._get_manager(target_class)
-
-        assert 'defaults' not in cls.FACTORY_DJANGO_GET_OR_CREATE, (
-            "'defaults' is a reserved keyword for get_or_create "
-            "(in %s.FACTORY_DJANGO_GET_OR_CREATE=%r)"
-            % (cls, cls.FACTORY_DJANGO_GET_OR_CREATE))
-
-        if cls.FACTORY_DJANGO_GET_OR_CREATE:
-            key_fields = {}
-            for field in cls.FACTORY_DJANGO_GET_OR_CREATE:
-                key_fields[field] = kwargs.pop(field)
-            key_fields['defaults'] = kwargs
-        else:
-            key_fields = kwargs
-
-        obj, _created = manager.get_or_create(*args, **key_fields)
-        return obj
-
-    @classmethod
-    def _after_postgeneration(cls, obj, create, results=None):
-        """Save again the instance if creating and at least one hook ran."""
-        if create and results:
-            # Some post-generation hooks ran, and may have modified us.
-            obj.save()
-
-
-class MogoFactory(Factory):
-    """Factory for mogo objects."""
-    ABSTRACT_FACTORY = True
-
-    @classmethod
-    def _build(cls, target_class, *args, **kwargs):
-        return target_class.new(*args, **kwargs)
-
-
 class BaseDictFactory(Factory):
     """Factory for dictionary-like classes."""
     ABSTRACT_FACTORY = True
@@ -675,3 +635,36 @@ def use_strategy(new_strategy):
         klass.FACTORY_STRATEGY = new_strategy
         return klass
     return wrapped_class
+
+
+class SQLAlchemyModelFactory(Factory):
+    """Factory for SQLAlchemy models. """
+
+    ABSTRACT_FACTORY = True
+    FACTORY_HIDDEN_ARGS=('SESSION',)
+
+    def __init__(self, session):
+        self.session = session
+
+    @classmethod
+    def _get_function(cls, function_name):
+        session = cls._declarations['SESSION']
+        sqlalchemy = __import__(session.__module__)
+        max = getattr(sqlalchemy.sql.functions, function_name)
+
+    @classmethod
+    def _setup_next_sequence(cls, *args, **kwargs):
+        """Compute the next available PK, based on the 'pk' database field."""
+        max = cls._get_function('max')
+        session = cls._declarations['SESSION']
+        pk = cls.FACTORY_FOR.__table__.primary_key.columns.values()[0].key
+        max_pk = session.query(max(getattr(cls.FACTORY_FOR, pk))).one()
+        return max_pk[0] + 1 if max_pk[0] else 1
+
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        """Create an instance of the model, and save it to the database."""
+        session = cls._declarations['SESSION']
+        obj = target_class(*args, **kwargs)
+        session.add(obj)
+        return obj
