@@ -57,6 +57,12 @@ class FakeModel(object):
             instance._defaults = defaults
             return instance, True
 
+        def create(self, **kwargs):
+            instance = FakeModel.create(**kwargs)
+            instance.id = 2
+            instance._defaults = None
+            return instance
+
         def values_list(self, *args, **kwargs):
             return self
 
@@ -119,7 +125,7 @@ class SimpleBuildTestCase(unittest.TestCase):
         self.assertEqual(obj.foo, 'bar')
 
     def test_create_custom_base(self):
-        obj = factory.create(FakeModel, foo='bar', FACTORY_CLASS=factory.DjangoModelFactory)
+        obj = factory.create(FakeModel, foo='bar', FACTORY_CLASS=factory.django.DjangoModelFactory)
         self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
@@ -135,7 +141,7 @@ class SimpleBuildTestCase(unittest.TestCase):
 
     def test_create_batch_custom_base(self):
         objs = factory.create_batch(FakeModel, 4, foo='bar',
-                FACTORY_CLASS=factory.DjangoModelFactory)
+                FACTORY_CLASS=factory.django.DjangoModelFactory)
 
         self.assertEqual(4, len(objs))
         self.assertEqual(4, len(set(objs)))
@@ -171,7 +177,7 @@ class SimpleBuildTestCase(unittest.TestCase):
 
     def test_generate_create_custom_base(self):
         obj = factory.generate(FakeModel, factory.CREATE_STRATEGY, foo='bar',
-                FACTORY_CLASS=factory.DjangoModelFactory)
+                FACTORY_CLASS=factory.django.DjangoModelFactory)
         self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
@@ -202,7 +208,7 @@ class SimpleBuildTestCase(unittest.TestCase):
 
     def test_generate_batch_create_custom_base(self):
         objs = factory.generate_batch(FakeModel, factory.CREATE_STRATEGY, 20, foo='bar',
-                FACTORY_CLASS=factory.DjangoModelFactory)
+                FACTORY_CLASS=factory.django.DjangoModelFactory)
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
@@ -232,7 +238,7 @@ class SimpleBuildTestCase(unittest.TestCase):
         self.assertEqual(obj.foo, 'bar')
 
     def test_simple_generate_create_custom_base(self):
-        obj = factory.simple_generate(FakeModel, True, foo='bar', FACTORY_CLASS=factory.DjangoModelFactory)
+        obj = factory.simple_generate(FakeModel, True, foo='bar', FACTORY_CLASS=factory.django.DjangoModelFactory)
         self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
@@ -258,7 +264,7 @@ class SimpleBuildTestCase(unittest.TestCase):
 
     def test_simple_generate_batch_create_custom_base(self):
         objs = factory.simple_generate_batch(FakeModel, True, 20, foo='bar',
-                FACTORY_CLASS=factory.DjangoModelFactory)
+                FACTORY_CLASS=factory.django.DjangoModelFactory)
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
@@ -293,7 +299,7 @@ class UsingFactoryTestCase(unittest.TestCase):
         test_object = TestObjectFactory.build()
         self.assertEqual(test_object.one, 'one')
 
-    def test_inheritance(self):
+    def test_inheriting_target_class(self):
         @factory.use_strategy(factory.BUILD_STRATEGY)
         class TestObjectFactory(factory.Factory, TestObject):
             FACTORY_FOR = TestObject
@@ -1247,7 +1253,7 @@ class IteratorTestCase(unittest.TestCase):
 
             @factory.iterator
             def one():
-                for i in range(10, 50):
+                for i in range(10, 50):  # pragma: no cover
                     yield i
 
         objs = TestObjectFactory.build_batch(20)
@@ -1292,7 +1298,7 @@ class BetterFakeModel(object):
 
 class DjangoModelFactoryTestCase(unittest.TestCase):
     def test_simple(self):
-        class FakeModelFactory(factory.DjangoModelFactory):
+        class FakeModelFactory(factory.django.DjangoModelFactory):
             FACTORY_FOR = FakeModel
 
         obj = FakeModelFactory(one=1)
@@ -1306,7 +1312,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
         class MyFakeModel(BetterFakeModel):
             objects = BetterFakeModelManager({'x': 1}, prev)
 
-        class MyFakeModelFactory(factory.DjangoModelFactory):
+        class MyFakeModelFactory(factory.django.DjangoModelFactory):
             FACTORY_FOR = MyFakeModel
             FACTORY_DJANGO_GET_OR_CREATE = ('x',)
             x = 1
@@ -1327,7 +1333,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
         class MyFakeModel(BetterFakeModel):
             objects = BetterFakeModelManager({'x': 1, 'y': 2, 'z': 3}, prev)
 
-        class MyFakeModelFactory(factory.DjangoModelFactory):
+        class MyFakeModelFactory(factory.django.DjangoModelFactory):
             FACTORY_FOR = MyFakeModel
             FACTORY_DJANGO_GET_OR_CREATE = ('x', 'y', 'z')
             x = 1
@@ -1348,7 +1354,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
         class MyFakeModel(BetterFakeModel):
             objects = BetterFakeModelManager({'x': 1}, prev)
 
-        class MyFakeModelFactory(factory.DjangoModelFactory):
+        class MyFakeModelFactory(factory.django.DjangoModelFactory):
             FACTORY_FOR = MyFakeModel
             FACTORY_DJANGO_GET_OR_CREATE = ('x',)
             x = 1
@@ -1369,7 +1375,7 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
         class MyFakeModel(BetterFakeModel):
             objects = BetterFakeModelManager({'x': 1, 'y': 2, 'z': 3}, prev)
 
-        class MyFakeModelFactory(factory.DjangoModelFactory):
+        class MyFakeModelFactory(factory.django.DjangoModelFactory):
             FACTORY_FOR = MyFakeModel
             FACTORY_DJANGO_GET_OR_CREATE = ('x', 'y', 'z')
             x = 1
@@ -1383,6 +1389,72 @@ class DjangoModelFactoryTestCase(unittest.TestCase):
         self.assertEqual(4, obj.z)
         self.assertEqual(2, obj.id)
 
+    def test_sequence(self):
+        class TestModelFactory(factory.django.DjangoModelFactory):
+            FACTORY_FOR = TestModel
+
+            a = factory.Sequence(lambda n: 'foo_%s' % n)
+
+        o1 = TestModelFactory()
+        o2 = TestModelFactory()
+
+        self.assertEqual('foo_2', o1.a)
+        self.assertEqual('foo_3', o2.a)
+
+        o3 = TestModelFactory.build()
+        o4 = TestModelFactory.build()
+
+        self.assertEqual('foo_4', o3.a)
+        self.assertEqual('foo_5', o4.a)
+
+    def test_no_get_or_create(self):
+        class TestModelFactory(factory.django.DjangoModelFactory):
+            FACTORY_FOR = TestModel
+
+            a = factory.Sequence(lambda n: 'foo_%s' % n)
+
+        o = TestModelFactory()
+        self.assertEqual(None, o._defaults)
+        self.assertEqual('foo_2', o.a)
+        self.assertEqual(2, o.id)
+
+    def test_get_or_create(self):
+        class TestModelFactory(factory.django.DjangoModelFactory):
+            FACTORY_FOR = TestModel
+            FACTORY_DJANGO_GET_OR_CREATE = ('a', 'b')
+
+            a = factory.Sequence(lambda n: 'foo_%s' % n)
+            b = 2
+            c = 3
+            d = 4
+
+        o = TestModelFactory()
+        self.assertEqual({'c': 3, 'd': 4}, o._defaults)
+        self.assertEqual('foo_2', o.a)
+        self.assertEqual(2, o.b)
+        self.assertEqual(3, o.c)
+        self.assertEqual(4, o.d)
+        self.assertEqual(2, o.id)
+
+    def test_full_get_or_create(self):
+        """Test a DjangoModelFactory with all fields in get_or_create."""
+        class TestModelFactory(factory.django.DjangoModelFactory):
+            FACTORY_FOR = TestModel
+            FACTORY_DJANGO_GET_OR_CREATE = ('a', 'b', 'c', 'd')
+
+            a = factory.Sequence(lambda n: 'foo_%s' % n)
+            b = 2
+            c = 3
+            d = 4
+
+        o = TestModelFactory()
+        self.assertEqual({}, o._defaults)
+        self.assertEqual('foo_2', o.a)
+        self.assertEqual(2, o.b)
+        self.assertEqual(3, o.c)
+        self.assertEqual(4, o.d)
+        self.assertEqual(2, o.id)
+
 
 class PostGenerationTestCase(unittest.TestCase):
     def test_post_generation(self):
@@ -1574,6 +1646,45 @@ class PostGenerationTestCase(unittest.TestCase):
         self.assertEqual(4, related.two)
 
 
+class RelatedFactoryExtractionTestCase(unittest.TestCase):
+    def setUp(self):
+        self.relateds = []
+
+        class TestRelatedObject(object):
+            def __init__(subself, obj):
+                self.relateds.append(subself)
+                subself.obj = obj
+                obj.related = subself
+
+        class TestRelatedObjectFactory(factory.Factory):
+            FACTORY_FOR = TestRelatedObject
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = factory.RelatedFactory(TestRelatedObjectFactory, 'obj')
+
+        self.TestRelatedObject = TestRelatedObject
+        self.TestRelatedObjectFactory = TestRelatedObjectFactory
+        self.TestObjectFactory = TestObjectFactory
+
+    def test_no_extraction(self):
+        o = self.TestObjectFactory()
+        self.assertEqual(1, len(self.relateds))
+        rel = self.relateds[0]
+        self.assertEqual(o, rel.obj)
+        self.assertEqual(rel, o.related)
+
+    def test_passed_value(self):
+        o = self.TestObjectFactory(one=42)
+        self.assertEqual([], self.relateds)
+        self.assertFalse(hasattr(o, 'related'))
+
+    def test_passed_none(self):
+        o = self.TestObjectFactory(one=None)
+        self.assertEqual([], self.relateds)
+        self.assertFalse(hasattr(o, 'related'))
+
+
 class CircularTestCase(unittest.TestCase):
     def test_example(self):
         sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
@@ -1761,54 +1872,5 @@ class ListTestCase(unittest.TestCase):
         ], o.two)
 
 
-class DjangoModelFactoryTestCase(unittest.TestCase):
-    def test_sequence(self):
-        class TestModelFactory(factory.DjangoModelFactory):
-            FACTORY_FOR = TestModel
-
-            a = factory.Sequence(lambda n: 'foo_%s' % n)
-
-        o1 = TestModelFactory()
-        o2 = TestModelFactory()
-
-        self.assertEqual('foo_2', o1.a)
-        self.assertEqual('foo_3', o2.a)
-
-        o3 = TestModelFactory.build()
-        o4 = TestModelFactory.build()
-
-        self.assertEqual('foo_4', o3.a)
-        self.assertEqual('foo_5', o4.a)
-
-    def test_no_get_or_create(self):
-        class TestModelFactory(factory.DjangoModelFactory):
-            FACTORY_FOR = TestModel
-
-            a = factory.Sequence(lambda n: 'foo_%s' % n)
-
-        o = TestModelFactory()
-        self.assertEqual({}, o._defaults)
-        self.assertEqual('foo_2', o.a)
-        self.assertEqual(2, o.id)
-
-    def test_get_or_create(self):
-        class TestModelFactory(factory.DjangoModelFactory):
-            FACTORY_FOR = TestModel
-            FACTORY_DJANGO_GET_OR_CREATE = ('a', 'b')
-
-            a = factory.Sequence(lambda n: 'foo_%s' % n)
-            b = 2
-            c = 3
-            d = 4
-
-        o = TestModelFactory()
-        self.assertEqual({'c': 3, 'd': 4}, o._defaults)
-        self.assertEqual('foo_2', o.a)
-        self.assertEqual(2, o.b)
-        self.assertEqual(3, o.c)
-        self.assertEqual(4, o.d)
-        self.assertEqual(2, o.id)
-
-
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     unittest.main()
